package cn.com.qmth.examcloud.service.core.service;
import static org.springframework.data.domain.ExampleMatcher.GenericPropertyMatchers.contains;
import static org.springframework.data.domain.ExampleMatcher.GenericPropertyMatchers.startsWith;

import java.io.InputStream;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Example;
import org.springframework.data.domain.ExampleMatcher;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import cn.com.qmth.examcloud.common.util.excel.ExcelError;
import cn.com.qmth.examcloud.common.util.excel.ExcelReader;
import cn.com.qmth.examcloud.common.util.excel.ExcelReaderHandle;
import cn.com.qmth.examcloud.service.core.entity.Org;
import cn.com.qmth.examcloud.service.core.entity.User;
import cn.com.qmth.examcloud.service.core.repo.OrgRepo;
import cn.com.qmth.examcloud.service.core.repo.UserRepo;

@Service
public class OrgService {

    @Autowired
    OrgRepo orgRepo;
    @Autowired
    UserRepo userRepo;
    
    @Transactional
	public List<ExcelError> importLearnCenter(Long orgId, InputStream inputStream) {
		List<Org> list = new ArrayList<Org>();
		Org parentOrg = orgRepo.findOne(orgId);
		ExcelReader excelReader = new ExcelReader(Org.class);
        List<ExcelError> excelErrors = excelReader.reader(inputStream, new ExcelReaderHandle() {
            @Override
            public ExcelError handle(Object obj) {
            	Org dto = (Org) obj;
                dto.setRootId(orgId);
                dto.setParentId(orgId);
                dto.setApps(parentOrg.getApps());
                dto.setCreateTime(new Date());
                dto.setEnable(true);
                ExcelError error = importCheck(dto);
                if (error == null) {
                	list.add(dto);
                }
                return error;
            }
        });
        orgRepo.save(list);
		return excelErrors;
	}

	private ExcelError importCheck(Org dto) {
		Org org = orgRepo.findByParentIdAndCode(dto.getParentId(),dto.getCode());
		if(org!=null){
			return new ExcelError("代码已存在");
		}
		return null;
	}

	/**
	 * 启用/禁用机构；禁用机构时禁用该机构下的所有用户
	 * @param id
	 * @param enable
	 * @return
	 */
	@Transactional
	public Org enableSchool(Long id, boolean enable) {
		Org org = orgRepo.findOne(id);
		org.setEnable(enable);
		orgRepo.save(org);
		if(enable == false){
			List<User> users = userRepo.findByOrgId(id);
			for (User user : users) {
				user.setEnable(false);
				userRepo.save(user);
			}
		}
		return org;
	}

	public Page<Org> findAll(Org orgCriteria, Pageable pageable) {
		ExampleMatcher exampleMatcher = ExampleMatcher.matching()
				.withMatcher("name", contains())
				.withMatcher("code", contains());
		Example<Org> examExamStudentple = Example.of(orgCriteria,
				exampleMatcher);
		return orgRepo.findAll(examExamStudentple, pageable);
	}

	public List<Org> findAll(Org orgCriteria) {
		ExampleMatcher exampleMatcher = ExampleMatcher.matching()
				.withMatcher("name", contains())
				.withMatcher("code", contains());
        Example<Org> examExample = Example.of(orgCriteria,exampleMatcher);
        return orgRepo.findAll(examExample);
	}

}
